/*
 * SPDX-License-Identifier: BSD-2-Clause
 *
 * Copyright (c) 2023 Andes Technology Corporation
 */

#ifndef _ANDES_HPM_H_
#define _ANDES_HPM_H_

static inline int andes_pmu_setup(void) { return 0; }

#endif /* _ANDES_HPM_H_ */
